
import os
import pandas as pd
from spomato.parsers import parse_album, parse_playlist, parse_saved_tracks

file_path = os.path.expanduser('~')+'/'

def cache_data(data,profile):
    data.to_csv(file_path+profile,index=False)

def load_cached_data(profile):
    data = pd.read_csv(file_path+profile)
    return data

def get_data(current_user_id=None,sp=None,profile_name=None,save=True,sourcetype='savedtracks',sourcelist=[],reset=False):
    if profile_name:
        if os.path.isfile(file_path+profile_name) and reset==False:
            data = load_cached_data(profile_name)
            # print 'loading from csv'
        else:
            data = get_new_data(current_user_id=current_user_id,sp=sp,sourcetype=sourcetype,sourcelist=sourcelist)
            # print 'loading data under profile'
            if save:
                cache_data(data,profile_name)
                # print 'caching data'

    else:

        data = get_new_data(current_user_id=current_user_id,sp=sp,sourcetype=sourcetype,sourcelist=sourcelist)
        # print 'just getting data'

    return data

def get_new_data(current_user_id,sp=None,sourcetype='savedtracks',sourcelist=[], market='US'):

    if sourcetype == 'savedtracks':
        # print 'SAVEDTRACKS'
        data = get_saved_tracks(sp)

    elif sourcetype == 'playlist':
        # print 'PLAYLISTS'
        playlist_df = get_playlists(sp)
        subpdf = playlist_df[playlist_df.playlist_name.isin(sourcelist)]
        dflist=[]
        for pl_id in subpdf.playlist_id:
            pl_json=sp.user_playlist(current_user_id,pl_id)
            pl_df = parse_playlist(pl_json)
            dflist.append(pl_df)

        data = pd.concat(dflist)
        data.drop_duplicates(inplace=True)

    elif sourcetype == 'artist':
        # print 'ARTISTS'
        dflist=[]
        for artist in sourcelist:
            artist_songs = get_artist_data(sp,artist)
            dflist.append(artist_songs)

        data = pd.concat(dflist)
        data.drop_duplicates(inplace=True)

    return data

def get_saved_tracks(read_sp):
    end=False
    i=0
    dflist = []
    while not end:
        data = read_sp.current_user_saved_tracks(limit=50,offset=i*50)['items']
        if len(data)>0:
            df = parse_saved_tracks(data)
            dflist.append(df)
            i+=1
        else:
            end=True


    df = pd.concat(dflist).reset_index(drop=True)

    return df

def pick_songs(df, time=25, extra=5, time_limit=None):

    time = time*60
    extra = extra*60
    if time_limit == None:
        time_limit = time/3.0
    subdf = df[df.time<=time_limit]
    time_used = 0

    song_list = []
    done = False
    while not done:

        subdf2 = subdf[subdf.time<=(time+extra-time_used)]

        if time_used > time:
            done = True

        elif len(subdf2)==0:
            done=True
        else:
            song = subdf2.sample().iloc[0]
            df = df[df.song_id!=song.song_id]
            subdf = subdf[subdf.song_id!=song.song_id]

            song_list.append(song)
            time_used += song.time

    songdf = pd.concat(song_list,axis=1).T

    return songdf, df



def get_artist_data(sp,artist_id):
    artist_albums=sp.artist_albums(artist_id)
    album_ids = [x['id'] for x in artist_albums['items']]
    album_jsons = sp.albums(album_ids)['albums']
    songdf = []
    for album in album_jsons:
        songs=parse_album(album)
        songdf.append(songs)
    data = pd.concat(songdf)
    return data


def make_playlist(sp, playlist_name,songdf,current_user_id,playlist_df=None):
    if playlist_df==None:
        playlist_df = get_playlists(sp)

    if playlist_name in playlist_df.playlist_name.tolist():
        # print('replacing')
        playlist_id = playlist_df[playlist_df.playlist_name==playlist_name].iloc[0].playlist_id
        sp.user_playlist_replace_tracks(user = current_user_id,
                                            playlist_id = playlist_id,
                                            tracks = songdf.song_id.tolist()
                                           )
    else:
        # print('new!')
        sp.user_playlist_create(current_user_id, playlist_name, public=False)
        playlist_df = get_playlists(sp)
        playlist_id = playlist_df[playlist_df.playlist_name==playlist_name].iloc[0].playlist_id
        sp.user_playlist_add_tracks(current_user_id,playlist_id,tracks = songdf.song_id.tolist())


def artist_id_search(sp, artist,limit=10,offset=0):
    ooo=sp.search(artist,type='artist',limit=limit,offset=offset)['artists']['items']

    if len(ooo)>0:
        index=['artist','id']
        df = pd.concat([pd.Series([x['name'],x['id']],index=index) for x in ooo],axis=1).T
    else:
        df=pd.DataFrame(columns=['name','id'])
    return df

def get_playlists(mod_sp):
    pl_json = mod_sp.current_user_playlists()
    playlist_list = [pd.Series([pl['name'],pl['id']],index=['playlist_name','playlist_id']) for pl in pl_json['items']]
    playlist_df = pd.concat(playlist_list,axis=1).T
    return playlist_df



import os
import pandas as pd
import spotipy
from spotipy import oauth2
from splinter import Browser
import requests
import base64
import six
import time


file_path = os.path.expanduser('~')+'/'

def get_token(username=None,password=None,login='facebook'):

    if os.path.isfile(file_path+'token_info.csv'):
        token_info = pd.Series.from_csv(file_path+'token_info.csv')
        token = token_info['access_token']
        # print 'reading token from csv'
        if int(token_info['expires_at'])-int(time.time()) <= 0:
            # print 'token expired, create new token'
            token,token_info = get_new_token(username,password,login)
            token_info.to_csv(file_path+'token_info.csv')

    else:
        # print 'generate first token'
        token,token_info = get_new_token(username,password,login)
        token_info.to_csv(file_path+'token_info.csv')

    return token,token_info

def get_new_token(username=None,password=None,login='facebook'):

    OAUTH_TOKEN_URL = 'https://accounts.spotify.com/api/token'

    client_id = os.getenv('SPOTIPY_CLIENT_ID')
    client_secret = os.getenv('SPOTIPY_CLIENT_SECRET')
    redirect_uri = os.getenv('SPOTIPY_REDIRECT_URI')
    username = os.getenv('SPOMATO_USERNAME')
    password = os.getenv('SPOMATO_PASSWORD')
    # username='astromars42@gmail.com'


    read_scope = 'playlist-read-private'
    modify_scope = 'playlist-modify-private'
    scope = read_scope + ' ' + modify_scope
    norm_scope = _normalize_scope(scope)

    sp_oauth = oauth2.SpotifyOAuth(client_id, client_secret, redirect_uri,
            scope=norm_scope, cache_path=".cache-" + username )
    auth_url = sp_oauth.get_authorize_url()

    code = get_url_code(auth_url,username,password,login)

    payload = {'redirect_uri': redirect_uri,
                   'code': code,
                   'grant_type': 'authorization_code',
                   'scope':norm_scope}

    headers = _make_authorization_headers(client_id,client_secret)

    response = requests.post(OAUTH_TOKEN_URL, data=payload,
            headers=headers, verify=True, proxies=None)

    token_info = response.json()

    token_info = _add_custom_values_to_token_info(token_info, norm_scope)

    mytoken = token_info['access_token']

    token_info = pd.Series(token_info)

    return mytoken,token_info

def get_url_code(auth_url,username,password,login='facebook'):
    b = Browser(driver_name='chrome')
    b.visit(auth_url)
    # removing  login click as it redirects to that page automatically now
    # b.click_link_by_partial_href("/en/login")
    if login=='facebook':
        b.click_link_by_partial_href("https://www.facebook.com")
        b.fill_form({'email':username,'pass':password})
        b.click_link_by_id('loginbutton')
    elif login=='spotify':
        b.fill_form({'username':username,'password':password})
        loginbutton=b.find_by_text('Log In')[0]
        loginbutton.click()
    b.visit(auth_url)
    codeurl = b.url
    code = codeurl.split("?code=")[1].split('&')[0]
    b.quit()

    return code

def _make_authorization_headers(client_id, client_secret):
    auth_header = base64.b64encode(six.text_type(client_id + ':' + client_secret).encode('ascii'))
    return {'Authorization': 'Basic %s' % auth_header.decode('ascii')}

def _normalize_scope(scope):
    if scope:
        scopes = scope.split()
        scopes.sort()
        return ' '.join(scopes)
    else:
        return None

def _add_custom_values_to_token_info(token_info, scope):
    '''
    Store some values that aren't directly provided by a Web API
    response.
    '''
    token_info['expires_at'] = int(time.time()) + token_info['expires_in']
    token_info['scope'] = scope
    return token_info



from setuptools import setup

def readme():
    with open('README.md') as f:
        return f.read()


major = 0
minor = 1
patch = 1
version = '.'.join([str(v) for v in [major, minor, patch]])


setup(name='spomato',
      version=version,
      description='Tomato Timer with Spotify',
      url='https://github.com/pm8k/spomato',
      download_url = 'https://github.com/pm8k/spomato/archive/0.1.tar.gz', # I'll explain this in a second

      long_description=readme(),
      author='Matthew Russell',
      author_email='astromars42@gmail.com',
      license='MIT',
      keywords=['python', 'spotify', 'spotipy', 'tomato', 'timer', 'music'],

      packages=['spomato'],
        install_requires=[
          'spotipy',
          'splinter',
          'pandas'
      ],
      include_package_data=True,

      zip_safe=False)



import pandas as pd

def parse_album(data, market='US'):
    series_list = []
    data = data['tracks']['items']
    for i in range(len(data)):
        record = data[i]
        songid = record['id']
        markets = record['available_markets']
        time = record['duration_ms']/1000
        if market in markets:
            series = pd.Series([songid,time],index=['song_id','time'])
            series_list.append(series)
    if len(series_list)>0:
        df = pd.concat(series_list,axis=1).transpose()
    else:
        df = pd.DataFrame(columns=['song_id','time'])
    return df

def parse_playlist(data, market='US'):
    series_list = []
    data = data['tracks']['items']
    for i in range(len(data)):
        record = data[i]['track']
        songid = record['id']
        markets = record['available_markets']
        time = record['duration_ms']/1000
        if market in markets:
            series = pd.Series([songid,time],index=['song_id','time'])
            series_list.append(series)

    df = pd.concat(series_list,axis=1).transpose()

    return df

def parse_saved_tracks(data,market='US'):
    series_list = []
    for i in range(len(data)):
        record = data[i]['track']
        songid = record['id']
        markets = record['available_markets']
        time = record['duration_ms']/1000
        if market in markets:
            series = pd.Series([songid,time],index=['song_id','time'])
            series_list.append(series)

    df = pd.concat(series_list,axis=1).transpose()

    return df
